<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class ErpController extends \Zeedhi\Framework\Controller\Simple {

    protected $erpController;
    
    public function __construct(\Service\ErpService $erpService) {
        $this->erpService = $erpService;
    }

	// INICIO NOVO LOGITEM
	public function getMaxSeqLog(){
		return $this->erpService->getMaxSeqLog();
	}
	
	public function setHoraRecLog($seqMaxLog){
		$this->erpService->setHoraRecLog($seqMaxLog);
	}
	 
	public function geraLogItem($seqMaxLog, $cdlog, $cddados){
		return $this->erpService->geraLogItem($seqMaxLog, $cdlog, $cddados);
	}
	
	public function setHoraRecLogItem($seqLog, $cddados){
		$this->erpService->setHoraRecLogItem($seqLog, $cddados);
	}
	
	/********************INICIO METODOS SERVER**************/
	
	const CADASTRO_PRODUTO_REQUIRED_FIELDS = array('CDPRODINT', 'DSNIVEL', 'NMPRODUTO', 'SGUNIDADE', 'VRFATCORR', 'VRFATOCONV', 'IDPRODESTO', 'IDPRODATIVO', 'IDACAO');
	
	private function validaCamposObrigatorios($paramArray, $requiredFields){
		$missingFields = '';
		
		$paramArrayKeys = array();
		foreach($paramArray as $key => $value){
			array_push($paramArrayKeys, $key);
		}
		
		$missingFieldsArray = array_filter($requiredFields, function($key) use ($paramArrayKeys){
			return !in_array($key, $paramArrayKeys);
		});
		foreach($missingFieldsArray as $field){
			$missingFields = empty($missingFields)?''.$field : $missingFields.', '.$field;
		}
		
		return $missingFields;
	}
		
    public function CadastraProduto($objXml) {
		try {
			$arrXml = $this->obj_to_array($objXml)['CadProd'];
			$xml = $this->arrayToXml($arrXml, 'CadastraProduto');
	
			$missingFields = $this->validaCamposObrigatorios($arrXml, self::CADASTRO_PRODUTO_REQUIRED_FIELDS);
			if($missingFields){
				$mensagem = 'Campos Obrigatórios: '.$missingFields;
				$this->erpService->gravaLogWebservice($xml, 'CadastraProduto', $arrXml, $mensagem);
				throw new \Exception($mensagem);
			}
			
			$produtoCadastrado = $this->erpService->getProduto($arrXml['CDPRODINT']);
			switch($produtoCadastrado){
				case 0: 
					$mensagem = $this->erpService->CadastraProduto($arrXml);
					break;
				case 1:
					$excluir = $arrXml['IDACAO'];// == 'EXCLUSAO' ? 'EXCLUSAO' : 'ALTERACAO'; // 3 - excluir | 2 - updatar
					$mensagem = $this->erpService->UpdateProduto($arrXml, $excluir);
					break;
				default:
					$mensagem = $produtoCadastrado;
					break;
			}
			
			if(!$mensagem){
				$mensagem = $this->erpService->CallProcedureImportaProduto();
			}
				
			
			if($mensagem){
				$this->erpService->gravaLogWebservice($xml, 'CadastraProduto', $arrXml, $mensagem);
				$response = array(
		    					'Status'=> 'E',
		    					'Mensagem'=> $mensagem
			    			);
				return $response;
				//$this->buildXml($response, 'CadastraProdutoResponse');
			}else{
			    $response = array(
	        					'Status'=> 'S'
	            			);
				return $response;
				//$this->buildXml($response, 'CadastraProdutoResponse');
			}
		}catch(\Exception $e){
			$mensagem = $e->getMessage();
			$this->erpService->gravaLogWebservice($xml, 'CadastraProduto', $arrXml, $mensagem);
			$return = array(
            					'Status'=> 'E',
            					'Mensagem'=> $e->getMessage()
            			);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $return));
			//$this->buildXml($response, 'CadastraProdutoResponse');
		}
	}
	
	public function CadastraProdutoJson(Request $request, Response $response) {
		try {
			
			if(empty($_POST)){
				$input = file_get_contents('php://input');
				$requestData = json_decode($input, true); 
			}else{
				$requestData = $_POST;
			}
			$dataSet = $requestData;
			
			$arrJson = $dataSet;
			$json = json_encode($arrJson);
	
			$missingFields = $this->validaCamposObrigatorios($arrJson, self::CADASTRO_PRODUTO_REQUIRED_FIELDS);

			if($missingFields){
				$mensagem = 'Campos Obrigatórios: '.$missingFields;
				$this->erpService->gravaLogWebservice($json, 'CadastraProduto', $arrJson, $mensagem);
				throw new \Exception($mensagem);
			}
			
			//$produtoCadastrado = $this->erpService->getProduto($arrJson['CDPRODINT']);
			$produtoCadastrado = 0;  //neste cliente duplica-se os registros na tabela de integração para manter o historico.
			switch($produtoCadastrado){
				case 0: 
					$mensagem = $this->erpService->CadastraProduto($arrJson);
					break;
				case 1:
					$excluir = $arrJson['IDACAO'];// == 'EXCLUSAO' ? 'EXCLUSAO' : 'ALTERACAO'; // 3 - excluir | 2 - updatar
					$mensagem = $this->erpService->UpdateProduto($arrJson, $excluir);
					break;
				default:
					$mensagem = $produtoCadastrado;
					break;
			}
			
			if(!$mensagem){
				$mensagem = $this->erpService->CallProcedureImportaProduto();
			}
				
			
			if($mensagem){
				$this->erpService->gravaLogWebservice($json, 'CadastraProduto', $arrJson, $mensagem);
				$return = array(
		    					'Status'=> 'E',
		    					'Mensagem'=> $mensagem
			    			);
			    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $return));
				//$this->buildXml($response, 'CadastraProdutoResponse');
			}else{
			    $return = array(
	        					'Status'=> 'S',
	        					'Mensagem'=> 'Integração Realizada com Sucesso.'
	            			);
	            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $return));
				//$this->buildXml($response, 'CadastraProdutoResponse');
			}
		}catch(\Exception $e){
			$mensagem = $e->getMessage();
			$this->erpService->gravaLogWebservice($json, 'CadastraProduto', $arrJson, $mensagem);
			$return = array(
            					'Status'=> 'E',
            					'Mensagem'=> $e->getMessage()
            			);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $return));
            //$response->setError(new Response\Error($e->getMessage(), $e->getCode()));
			//$this->buildXml($response, 'CadastraProdutoResponse');
		}
	}
	
	public function ConsultaProdutoJson(Request $request, Response $response){
		try{
			if(empty($_POST)){
				$input = file_get_contents('php://input');
				$requestData = json_decode($input, true); 
			}else{
				$requestData = $_POST;
			}
			
			$dataSet = $requestData;
			$arrJson = $dataSet;
			$json = json_encode($arrJson);
			if(!isset($arrJson['CDPRODUTO']))
				$arrJson['CDPRODUTO'] = '';
			if(!isset($arrJson['CDPRODINT']))
				$arrJson['CDPRODINT'] = '';
			if(!isset($arrJson['CDARVPROD']))
				$arrJson['CDARVPROD'] = '';
			
			$consultaProduto = $this->erpService->consultaProduto($arrJson['CDPRODUTO'], $arrJson['CDPRODINT'], $arrJson['CDARVPROD']);
			
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsultaProduto', $consultaProduto));
			
		}catch(\Exception $e){
			$mensagem = $e->getMessage();
			$this->erpService->gravaLogWebservice($json, 'CadastraProduto', $arrJson, $mensagem);
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
		}
	}
	
	public function getSolicitaCompra(Request $request, Response $response){
		if(empty($_POST)){
			$input = file_get_contents('php://input');
			$requestData = json_decode($input, true); 
		}else{
			$requestData = $_POST;
		}
		
		$json = json_encode($requestData);
		if(!isset($requestData['NRITEMSOLINI']))
			$requestData['NRITEMSOLINI'] = $this->erpService->getMinSoli();
		if(!isset($requestData['NRITEMSOLFIN']))
			$requestData['NRITEMSOLFIN'] = $this->erpService->getMaxSoli();
			
		if(!isset($requestData['NRITEMSOLINI']) || !isset($requestData['NRITEMSOLFIN']))
			throw new \Exception('NRITEMSOLINI e NRITEMSOLFIN devem ser informados.');
		
		
		$consultaSoliCompra = $this->erpService->getSoliCompra($requestData['NRITEMSOLINI'], $requestData['NRITEMSOLFIN']);
		
		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getSolicompra', $consultaSoliCompra));
		
		/*
		$stringIn = '';
		foreach($consultaSoliCompra as $soli){
			if(!$stringIn)
				$stringIn = $stringIn.$soli['NRITEMSOLI'];
			else
				$stringIn = $stringIn.','.$soli['NRITEMSOLI'];
		}
		*/
		
		//$this->erpService->updateSoliCompra($requestData['NRITEMSOLINI'], $requestData['NRITEMSOLFIN']);
		
	}
	
	private function obj_to_array($data){
	    if (is_array($data) || is_object($data)){
	        $result = array();
	        foreach ($data as $key => $value){
	            $result[$key] = $this->obj_to_array($value);
	        }
	        return $result;
	    }
	    return $data;
	}

	private function buildXml($data, $method){
		// coloca no formato XML
		$xml = $this->arrayToXml($data, $method);
		return $xml;
	}
	
	private function arrayToXml($data, $elementName) {
		$result = '';
	    foreach($data as $key => $value) {
	        if (is_array($value)) {
	        	if(is_numeric($key) || $key == 'NFUnica' || $key == 'SEPUnica'){
	        		$result .= $this->arrayToXml($value,'ITEM'.$elementName);
	        	}else{
	        		$result .= $this->arrayToXml($value, $key);
	        	}
	        } else {
	        	$result .= '<'.$key.'>'.  $value . '</'.$key.'>';
	        }
	    }
	    if ($elementName) {
	     	return '<'.$elementName.'>'.  $result. '</'.$elementName.'>';
	    } else {
	    	return $result;
	    }
	}
	
	private function arrayToObject($data){
		$CDLAYOUT = $data['CDLAYOUT'];
		$ARRAY_REDUCAOZ = $data['REDUCAOZ']; 
		$ARRAY_IMPOSTO = $data['IMPOSTO'];
		$ARRAY_CUPOM = $data['CUPOM'];
		$ARRAY_ITEMS = $data['ITEMS'];
		$ARRAY_SANGRIA = $data['SANGRIA'];
		$ARRAY_RECEBIMENTO = $data['RECEBIMENTO'];
		$data_object = new DATA($CDLAYOUT,$ARRAY_REDUCAOZ,$ARRAY_CUPOM,$ARRAY_ITEMS,$ARRAY_RECEBIMENTO,$ARRAY_IMPOSTO,$ARRAY_SANGRIA);
		return $data_object;
	}
	
	public function array_some(array $array, callable $fn) {
        foreach ($array as $value) {
            if($fn($value)) {
                return true;
            }
        }
        return false;
    }
    
	public function previsaoConsumo(Request $request, Response $response) {
		try {
			
			if(empty($_POST)){
				$input = file_get_contents('php://input');
				$requestData = json_decode($input, true); 
			}else{
				$requestData = $_POST;
			}
			$dataSet = $requestData;
			
            $previsaoInsumo = $this->erpService->previsaoConsumo($dataSet['CDEXFILIAL'],
            													$dataSet['DTINIPREVCONS'],
            													$dataSet['DTFIMPREVCONS'],
            													$dataSet['CDSERVFILI']);
            
			if(!empty($previsaoInsumo)){
				$dataSet = array(
					'error' => false,
					'data'  => $previsaoInsumo
				);
			}else{
				$dataSet = array(
	                'error' => true,
	                'message' => 'Nenhum dado encontrado para a '.
	                			'Filial: '.$dataSet['CDEXFILIAL'].
	                			', entre as datas de: '.$dataSet['DTINIPREVCONS'].
	                			' e '.$dataSet['DTFIMPREVCONS']
	                				
	            );
			}
		
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $dataSet));
				
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function importaMoviest(Request $request, Response $response) {
		try {   
			
			if(empty($_POST)){
				$input = file_get_contents('php://input');
				$requestData = json_decode($input, true); 
			}else{
				$requestData = $_POST;
			}
			$dataSet = $requestData;

			$arrJson = $dataSet;
			$json = json_encode($arrJson);
			
            $importaEntrada = $this->erpService->importaEntrada($dataSet);
            if(!isset($importaEntrada['P_MENSAGEMERRO'])){
            	$bkpError = $importaEntrada;
            	$importaEntrada = array();
            	$importaEntrada['P_MENSAGEMERRO'] = $bkpError;
            }
            	
			if($importaEntrada['P_MENSAGEMERRO'] == 'S'){
				$dataSet = array(
		    					'Status'=> 'S',
		    					'Mensagem'=> 'Importado com sucesso'
			    			);
			}else{
				$dataSet = array(
		    					'Status'=> 'E',
		    					'Mensagem'=> $importaEntrada['P_MENSAGEMERRO']
			    			);
			}
		
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $dataSet));
				
        }catch(\Exception $e){
			$mensagem = $e->getMessage();
			$this->erpService->gravaLogWebservice($json, 'importaMoviest', $arrJson, $mensagem);
			$return = array(
            					'Status'=> 'E',
            					'Mensagem'=> $e->getMessage()
            			);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('response', $return));
            //$response->setError(new Response\Error($e->getMessage(), $e->getCode()));
			//$this->buildXml($response, 'CadastraProdutoResponse');
		}
    }
    
}