/*
Descrio: 

Procedure destinada ao clculo da alquota do simples nacional

Parmetros:

(IN) P_CDEMPRESA - Cdigo da empresa
(IN) P_CDTIPOOPER - Cdigo do tipo de operao
(IN) P_INIVALID - Data que ser utilizada para o clculo da alquota
(IN) P_CDFISOPEITEM - Cdigo fiscal da operao (sem o 1 dgito do CFOP)
(IN) P_CDIMPOSTONFE - Cdigo do imposto NF-e
(IN) P_VRSUBLIMITESN - Valor do sublimite da empresa
(IN) P_RATEIODIFERENCA - identifica se o processo ser utilizado para clculo da alquota ou apenas para validar a diferena entre a aliquota efetiva e o 
(OUT) P_RBT12 - Retorno da receita bruta total do ltimos 12 meses
(OUT) P_RBA - Retorno da receita bruta acumulada no ano corrente
(OUT) P_CDSIMPLESNACIONAL - Retorno da tabela do simples nacional que a empresa se enquadra
(OUT) P_CDFAIXA - Retorno da faixa de valores da tabela do simples nacional que a empresa se enquadra
(OUT) P_VRALIQUOTASN - Retorno da alquota do imposto

Criao:

Gleydson Israel da Silva Gazola

Observaes:

Em resumo o simples nacional se define na seguinte frmula:

AE = (RBT12 x AliqNo - PD) / RBT12

Onde:
AE = alquota efetiva
RBT12 = receita bruta total do ltimos 12 meses
AliqNo = alquota nominal da faixa
PD = parcela a deduzir

Aps encontrar a aliquota efetiva  s calcular a repartio dos tributos com a seguinte frmula:

Trib = AE * AliqTrib

Onde:
AliqFinal = Alquota final do imposto
AE = Alquota efetiva
AliqTrib = Alquota nominal do imposto
*/

CREATE OR REPLACE PROCEDURE CALCULA_ALIQUOTA_SN(
   P_CDEMPRESA IN VARCHAR2,
   P_CDTIPOOPER IN VARCHAR2,
   P_INIVALID IN DATE,
   P_CDFISOPEITEM IN VARCHAR2,
   P_CDIMPOSTONFE IN VARCHAR2,
   P_VRSUBLIMITESN IN FLOAT,
   P_RATEIODIFERENCA IN VARCHAR2,
   P_RBT12 OUT FLOAT,
   P_RBA OUT FLOAT,
   P_CDSIMPLESNACIONAL OUT VARCHAR2,
   P_CDFAIXA OUT VARCHAR2,
   P_VRALIQUOTASN OUT FLOAT)
AS
   /* VARIVEIS DA 1 FAIXA */
   V_CD1FAIXA VARCHAR2(05);
   V_VRALIQSIMPLES1FAIXA FLOAT;
   V_VRPARCDEDUZIR1FAIXA FLOAT;
   V_VRALIQIMPOSTO1FAIXA FLOAT;
   V_VRPERRATISS1FAIXA FLOAT;
   V_VRALIQEFETIVA1FAIXA FLOAT;
   /* VARIVEIS DA 2 FAIXA */
   V_CD2FAIXA VARCHAR2(05);
   V_VRALIQSIMPLES2FAIXA FLOAT;
   V_VRPARCDEDUZIR2FAIXA FLOAT;
   V_VRALIQIMPOSTO2FAIXA FLOAT;
   V_VRPERRATISS2FAIXA FLOAT;
   V_VRALIQEFETIVA2FAIXA FLOAT;
   /* VARIVEIS DA 3 FAIXA */
   V_CD3FAIXA VARCHAR2(05);
   V_VRALIQSIMPLES3FAIXA FLOAT;
   V_VRPARCDEDUZIR3FAIXA FLOAT;
   V_VRALIQIMPOSTO3FAIXA FLOAT;
   V_VRPERRATISS3FAIXA FLOAT;
   V_VRALIQEFETIVA3FAIXA FLOAT;
   /* VARIVEIS DA 4 FAIXA */
   V_CD4FAIXA VARCHAR2(05);
   V_VRALIQSIMPLES4FAIXA FLOAT;
   V_VRPARCDEDUZIR4FAIXA FLOAT;
   V_VRALIQIMPOSTO4FAIXA FLOAT;
   V_VRPERRATISS4FAIXA FLOAT;
   V_VRALIQEFETIVA4FAIXA FLOAT;
   /* VARIVEIS DA 5 FAIXA */
   V_CD5FAIXA VARCHAR2(05);
   V_VRALIQSIMPLES5FAIXA FLOAT;
   V_VRPARCDEDUZIR5FAIXA FLOAT;
   V_VRALIQIMPOSTO5FAIXA FLOAT;
   V_VRPERRATISS5FAIXA FLOAT;
   V_VRALIQEFETIVA5FAIXA FLOAT;
   /* VARIVEIS DA 6 FAIXA */
   V_CD6FAIXA VARCHAR2(05);
   V_VRALIQSIMPLES6FAIXA FLOAT;
   V_VRPARCDEDUZIR6FAIXA FLOAT;
   V_VRALIQIMPOSTO6FAIXA FLOAT;
   V_VRPERRATISS6FAIXA FLOAT;
   V_VRALIQEFETIVA6FAIXA FLOAT;
   /* VARIVEIS AUXILIARES DAS FAIXAS */
   V_CDFAIXAAUX VARCHAR2(05);
   V_VRALIQSIMPLESFAIXAAUX FLOAT;
   V_VRPARCDEDUZIRFAIXAAUX FLOAT;
   V_VRALIQIMPOSTOFAIXAAUX FLOAT;
   V_VRPERRATISSFAIXAAUX FLOAT;
   V_VRALIQEFETIVAFAIXAAUX FLOAT;

   V_FAIXAENQUADRADA NUMBER;
   V_CDSIMPLESNACIONAL VARCHAR2(05);
   V_RBT12 FLOAT; 
   V_RBA FLOAT; 
   V_QTDEMESES NUMBER;
   V_VRALIQIMPOSTOISS FLOAT;
   V_VRALIQIMPOSTOISSAUX FLOAT;
   V_VRALIQIMPOSTO FLOAT;
   V_CDIMPOSTOMAXALIQUOTA VARCHAR2(02);
   V_VRALIQSIMPLESRATEIO FLOAT;
   V_RATEIADIFISS VARCHAR2(01);
   V_VRALIQSIMPLESISSAUX FLOAT;
   V_RBT12VALIDA FLOAT;
   V_RBAVALIDA FLOAT;
   V_CDSIMPLESNACIONALVALIDA VARCHAR2(05);
   V_CDFAIXAVALIDA VARCHAR2(05);
   V_VRALIQUOTASNVALIDA FLOAT;
BEGIN

   V_RATEIADIFISS := 'N';

   BEGIN
      SELECT MAX(C.CDSIMPLESNACIONAL)
        INTO V_CDSIMPLESNACIONAL
        FROM CFOPSIMPLESNAC C, SIMPLESNACIONAL S
       WHERE C.CDCFOPPADRAO = P_CDFISOPEITEM
         AND C.CDSIMPLESNACIONAL = S.CDSIMPLESNACIONAL
         AND S.DTINIVIGENCIA <= P_INIVALID
         AND NVL(S.DTFIMVIGENCIA, P_INIVALID) >= P_INIVALID;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_CDSIMPLESNACIONAL := NULL;
   END;

   IF V_CDSIMPLESNACIONAL IS NULL THEN
     BEGIN
        SELECT MAX(S.CDSIMPLESNACIONAL)
          INTO V_CDSIMPLESNACIONAL
          FROM SIMPLESNACIONAL S, EMPRSIMPLESNAC E, TPOPERSIMPLESNAC T
         WHERE S.CDSIMPLESNACIONAL = E.CDSIMPLESNACIONAL
           AND E.CDEMPRESA = P_CDEMPRESA
           AND S.CDSIMPLESNACIONAL = T.CDSIMPLESNACIONAL
           AND T.CDTIPOOPER = P_CDTIPOOPER
           AND T.IDENTRSAIDOP = 'S'
           AND S.DTINIVIGENCIA  <= P_INIVALID
           AND NVL(S.DTFIMVIGENCIA, P_INIVALID) >= P_INIVALID;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_CDSIMPLESNACIONAL := NULL;
      END;
   END IF;

   IF V_CDSIMPLESNACIONAL IS NOT NULL THEN
      V_FAIXAENQUADRADA := 0;
      V_QTDEMESES := 0;

      --RECEITA BRUTA ACUMULADA NO ANO CORRENTE
      SELECT NVL(SUM(NVL(VRRECEITABR, 0)), 0)
        INTO V_RBA
        FROM RECEITABRUTA
       WHERE CDEMPRESA = P_CDEMPRESA
         AND DTREFRECEITABR BETWEEN TRUNC(TO_DATE('01/01/' || TO_CHAR(P_INIVALID, 'YYYY')), 'MONTH') AND LAST_DAY(ADD_MONTHS(P_INIVALID, -1)) /* NO CONSIDERA O MS ATUAL */
         AND IDSTATUSMES = 'B';

      --RECEITA BRUTA TOTAL DOS LTIMOS 12 MESES
      SELECT NVL(SUM(NVL(VRRECEITABR, 0)), 0)
        INTO V_RBT12
        FROM RECEITABRUTA
       WHERE CDEMPRESA = P_CDEMPRESA
         AND DTREFRECEITABR BETWEEN TRUNC(ADD_MONTHS(P_INIVALID, -12), 'MONTH') AND LAST_DAY(ADD_MONTHS(P_INIVALID, -1)) /* NO CONSIDERA O MS ATUAL */
         AND IDSTATUSMES = 'B';

      --SEGUNDO A LEGISLAO, CASO NO TENHA FATURAMENTO NOS LTIMOS 12 MESES CONSIDERAR VALOR 1
      IF V_RBT12 = 0 THEN
         V_RBT12 := 1;
      END IF;

      BEGIN
         SELECT COUNT(DISTINCT DTREFRECEITABR)
           INTO V_QTDEMESES
           FROM RECEITABRUTA
          WHERE CDEMPRESA = P_CDEMPRESA
            AND DTREFRECEITABR BETWEEN TRUNC(ADD_MONTHS(P_INIVALID, -12), 'MONTH') AND LAST_DAY(ADD_MONTHS(P_INIVALID,-1)) /* NO CONSIDERA O MS ATUAL */
            AND IDSTATUSMES = 'B';
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_QTDEMESES := 0;
      END;

      IF (V_QTDEMESES > 0) AND (V_QTDEMESES < 12) THEN
         V_RBT12 := ROUND((V_RBT12 / V_QTDEMESES) * 12, 2);
      END IF;

      --IDENTIFICANDO A FAIXA DE FATURAMENTO 
      IF V_RBT12 <= 180000 THEN
         V_FAIXAENQUADRADA := 1;
      ELSIF (V_RBT12 > 180000) AND (V_RBT12 <= 360000) THEN
         V_FAIXAENQUADRADA := 2;
      ELSIF (V_RBT12 > 360000) AND (V_RBT12 <= 720000) THEN
         V_FAIXAENQUADRADA := 3;
      ELSIF (V_RBT12 > 720000) AND (V_RBT12 <= 1800000) THEN
         V_FAIXAENQUADRADA := 4;
      ELSIF (V_RBT12 > 1800000) AND (V_RBT12 <= 3600000) THEN
         V_FAIXAENQUADRADA := 5;
      ELSIF (V_RBT12 > 3600000) AND (V_RBT12 <= 4800000) THEN
         V_FAIXAENQUADRADA := 6;
      ELSE
         V_FAIXAENQUADRADA := 0;
      END IF;

      IF V_FAIXAENQUADRADA = 1 THEN --CARREGANDO AS INFORMAES DA 1 FAIXA
         BEGIN
            SELECT F.CDFAIXA,
                   NVL(F.VRALIQSIMPLES, 0),
                   NVL(F.VRPARCDEDUZIR, 0),
                   NVL(I.VRALIQSIMPLES, 0),
                   NVL(I.VRPERRATISS, 0),
                   ((((V_RBT12 * NVL(F.VRALIQSIMPLES, 0) / 100) - NVL(F.VRPARCDEDUZIR, 0)) / CASE V_RBT12 WHEN 0 THEN 1 ELSE V_RBT12 END) * 100)
              INTO V_CD1FAIXA,
                   V_VRALIQSIMPLES1FAIXA,
                   V_VRPARCDEDUZIR1FAIXA,
                   V_VRALIQIMPOSTO1FAIXA,
                   V_VRPERRATISS1FAIXA,
                   V_VRALIQEFETIVA1FAIXA
              FROM FAIXASIMPLESNAC F JOIN IMPSIMPLESFAIXA I ON (F.CDSIMPLESNACIONAL = I.CDSIMPLESNACIONAL)
                                                           AND (F.CDFAIXA = I.CDFAIXA)
                                     JOIN IMPOSTO IP ON (I.CDIMPOSTO = IP.CDIMPOSTO)
             WHERE F.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
               AND IP.CDIMPOSTONFE = P_CDIMPOSTONFE
               AND TRUNC(F.VRINICIAL) = 0
               AND TRUNC(F.VRFINAL) = 180000;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CD1FAIXA := NULL;
               V_VRALIQSIMPLES1FAIXA := 0;
               V_VRPARCDEDUZIR1FAIXA := 0;
               V_VRALIQIMPOSTO1FAIXA := 0;
               V_VRPERRATISS1FAIXA := 0;
               V_VRALIQEFETIVA1FAIXA := 0;
         END;
      END IF;

      IF V_FAIXAENQUADRADA = 2 THEN --CARREGANDO AS INFORMAES DA 2 FAIXA
         BEGIN
            SELECT F.CDFAIXA,
                   NVL(F.VRALIQSIMPLES, 0),
                   NVL(F.VRPARCDEDUZIR, 0),
                   NVL(I.VRALIQSIMPLES, 0),
                   NVL(I.VRPERRATISS, 0),
                   ((((V_RBT12 * NVL(F.VRALIQSIMPLES, 0) / 100) - NVL(F.VRPARCDEDUZIR, 0)) / CASE V_RBT12 WHEN 0 THEN 1 ELSE V_RBT12 END) * 100)
              INTO V_CD2FAIXA,
                   V_VRALIQSIMPLES2FAIXA,
                   V_VRPARCDEDUZIR2FAIXA,
                   V_VRALIQIMPOSTO2FAIXA,
                   V_VRPERRATISS2FAIXA,
                   V_VRALIQEFETIVA2FAIXA
              FROM FAIXASIMPLESNAC F JOIN IMPSIMPLESFAIXA I ON (F.CDSIMPLESNACIONAL = I.CDSIMPLESNACIONAL)
                                                           AND (F.CDFAIXA = I.CDFAIXA)
                                     JOIN IMPOSTO IP ON (I.CDIMPOSTO = IP.CDIMPOSTO)
             WHERE F.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
               AND IP.CDIMPOSTONFE = P_CDIMPOSTONFE
               AND TRUNC(F.VRINICIAL) = 180000
               AND TRUNC(F.VRFINAL) = 360000;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CD2FAIXA := NULL;
               V_VRALIQSIMPLES2FAIXA := 0;
               V_VRPARCDEDUZIR2FAIXA := 0;
               V_VRALIQIMPOSTO2FAIXA := 0;
               V_VRPERRATISS2FAIXA := 0;
               V_VRALIQEFETIVA2FAIXA := 0;
         END;
      END IF;

      IF V_FAIXAENQUADRADA = 3 THEN --CARREGANDO AS INFORMAES DA 3 FAIXA
         BEGIN
            SELECT F.CDFAIXA,
                   NVL(F.VRALIQSIMPLES, 0),
                   NVL(F.VRPARCDEDUZIR, 0),
                   NVL(I.VRALIQSIMPLES, 0),
                   NVL(I.VRPERRATISS, 0),
                   ((((V_RBT12 * NVL(F.VRALIQSIMPLES, 0) / 100) - NVL(F.VRPARCDEDUZIR, 0)) / CASE V_RBT12 WHEN 0 THEN 1 ELSE V_RBT12 END) * 100)
              INTO V_CD3FAIXA,
                   V_VRALIQSIMPLES3FAIXA,
                   V_VRPARCDEDUZIR3FAIXA,
                   V_VRALIQIMPOSTO3FAIXA,
                   V_VRPERRATISS3FAIXA,
                   V_VRALIQEFETIVA3FAIXA
              FROM FAIXASIMPLESNAC F JOIN IMPSIMPLESFAIXA I ON (F.CDSIMPLESNACIONAL = I.CDSIMPLESNACIONAL)
                                                           AND (F.CDFAIXA = I.CDFAIXA)
                                     JOIN IMPOSTO IP ON (I.CDIMPOSTO = IP.CDIMPOSTO)
             WHERE F.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
               AND IP.CDIMPOSTONFE = P_CDIMPOSTONFE
               AND TRUNC(F.VRINICIAL) = 360000
               AND TRUNC(F.VRFINAL) = 720000;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CD3FAIXA := NULL;
               V_VRALIQSIMPLES3FAIXA := 0;
               V_VRPARCDEDUZIR3FAIXA := 0;
               V_VRALIQIMPOSTO3FAIXA := 0;
               V_VRPERRATISS3FAIXA := 0;
               V_VRALIQEFETIVA3FAIXA := 0;
         END;
      END IF;

      --QUANDO O FATURAMENTO DA EMPRESA ESTIVER NA 5 OU 6 FAIXA SER CARREGADO TAMBM O DADOS DA 4. ISTO PORQUE EXISTEM SITUAES EM QUE ELE SER UTILIZADO PARA REALIZAR ALGUM CLCULO
      IF V_FAIXAENQUADRADA IN (4, 5, 6) THEN --CARREGANDO AS INFORMAES DA 4 FAIXA
         BEGIN
            SELECT F.CDFAIXA,
                   NVL(F.VRALIQSIMPLES, 0),
                   NVL(F.VRPARCDEDUZIR, 0),
                   NVL(I.VRALIQSIMPLES, 0),
                   NVL(I.VRPERRATISS, 0),
                   ((((V_RBT12 * NVL(F.VRALIQSIMPLES, 0) / 100) - NVL(F.VRPARCDEDUZIR, 0)) / CASE V_RBT12 WHEN 0 THEN 1 ELSE V_RBT12 END) * 100)
              INTO V_CD4FAIXA,
                   V_VRALIQSIMPLES4FAIXA,
                   V_VRPARCDEDUZIR4FAIXA,
                   V_VRALIQIMPOSTO4FAIXA,
                   V_VRPERRATISS4FAIXA,
                   V_VRALIQEFETIVA4FAIXA
              FROM FAIXASIMPLESNAC F JOIN IMPSIMPLESFAIXA I ON (F.CDSIMPLESNACIONAL = I.CDSIMPLESNACIONAL)
                                                           AND (F.CDFAIXA = I.CDFAIXA)
                                     JOIN IMPOSTO IP ON (I.CDIMPOSTO = IP.CDIMPOSTO)
             WHERE F.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
               AND IP.CDIMPOSTONFE = P_CDIMPOSTONFE
               AND TRUNC(F.VRINICIAL) = 720000
               AND TRUNC(F.VRFINAL) = 1800000;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CD4FAIXA := NULL;
               V_VRALIQSIMPLES4FAIXA := 0;
               V_VRPARCDEDUZIR4FAIXA := 0;
               V_VRALIQIMPOSTO4FAIXA := 0;
               V_VRPERRATISS4FAIXA := 0;
               V_VRALIQEFETIVA4FAIXA := 0;
         END;
      END IF;

      --QUANDO O FATURAMENTO DA EMPRESA ESTIVER NA 6 FAIXA SER CARREGADO TAMBM O DADOS DA 5. ISTO PORQUE EXISTEM SITUAES EM QUE ELE SER UTILIZADO PARA REALIZAR ALGUM CLCULO
      IF V_FAIXAENQUADRADA IN (5, 6) THEN --CARREGANDO AS INFORMAES DA 5 FAIXA
         BEGIN
            SELECT F.CDFAIXA,
                   NVL(F.VRALIQSIMPLES, 0),
                   NVL(F.VRPARCDEDUZIR, 0),
                   NVL(I.VRALIQSIMPLES, 0),
                   NVL(I.VRPERRATISS, 0),
                   ((((V_RBT12 * NVL(F.VRALIQSIMPLES, 0) / 100) - NVL(F.VRPARCDEDUZIR, 0)) / CASE V_RBT12 WHEN 0 THEN 1 ELSE V_RBT12 END) * 100)
              INTO V_CD5FAIXA,
                   V_VRALIQSIMPLES5FAIXA,
                   V_VRPARCDEDUZIR5FAIXA,
                   V_VRALIQIMPOSTO5FAIXA,
                   V_VRPERRATISS5FAIXA,
                   V_VRALIQEFETIVA5FAIXA
              FROM FAIXASIMPLESNAC F JOIN IMPSIMPLESFAIXA I ON (F.CDSIMPLESNACIONAL = I.CDSIMPLESNACIONAL)
                                                           AND (F.CDFAIXA = I.CDFAIXA)
                                     JOIN IMPOSTO IP ON (I.CDIMPOSTO = IP.CDIMPOSTO)
             WHERE F.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
               AND IP.CDIMPOSTONFE = P_CDIMPOSTONFE
               AND TRUNC(F.VRINICIAL) = 1800000
               AND TRUNC(F.VRFINAL) = 3600000;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CD5FAIXA := NULL;
               V_VRALIQSIMPLES5FAIXA := 0;
               V_VRPARCDEDUZIR5FAIXA := 0;
               V_VRALIQIMPOSTO5FAIXA := 0;
               V_VRPERRATISS5FAIXA := 0;
               V_VRALIQEFETIVA5FAIXA := 0;
         END;
      END IF;

      IF V_FAIXAENQUADRADA = 6 THEN --CARREGANDO AS INFORMAES DA 6 FAIXA
         BEGIN
            SELECT F.CDFAIXA,
                   NVL(F.VRALIQSIMPLES, 0),
                   NVL(F.VRPARCDEDUZIR, 0),
                   NVL(I.VRALIQSIMPLES, 0),
                   NVL(I.VRPERRATISS, 0),
                   ((((V_RBT12 * NVL(F.VRALIQSIMPLES, 0) / 100) - NVL(F.VRPARCDEDUZIR, 0)) / CASE V_RBT12 WHEN 0 THEN 1 ELSE V_RBT12 END) * 100)
              INTO V_CD6FAIXA,
                   V_VRALIQSIMPLES6FAIXA,
                   V_VRPARCDEDUZIR6FAIXA,
                   V_VRALIQIMPOSTO6FAIXA,
                   V_VRPERRATISS6FAIXA,
                   V_VRALIQEFETIVA6FAIXA
              FROM FAIXASIMPLESNAC F JOIN IMPSIMPLESFAIXA I ON (F.CDSIMPLESNACIONAL = I.CDSIMPLESNACIONAL)
                                                           AND (F.CDFAIXA = I.CDFAIXA)
                                     JOIN IMPOSTO IP ON (I.CDIMPOSTO = IP.CDIMPOSTO)
             WHERE F.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
               AND IP.CDIMPOSTONFE = P_CDIMPOSTONFE
               AND TRUNC(F.VRINICIAL) = 3600000
               AND TRUNC(F.VRFINAL) = 4800000;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CD6FAIXA := NULL;
               V_VRALIQSIMPLES6FAIXA := 0;
               V_VRPARCDEDUZIR6FAIXA := 0;
               V_VRALIQIMPOSTO6FAIXA := 0;
               V_VRPERRATISS6FAIXA := 0;
               V_VRALIQEFETIVA6FAIXA := 0;
         END;
      END IF;

      --GRAVANDO EM UMA VARIVEL AUXILIAR OS DADOS DA FAIXA PRINCIPAL DO FATURAMENTO
      IF V_RBT12 > 0 THEN
         IF V_FAIXAENQUADRADA = 1 THEN
            V_CDFAIXAAUX := V_CD1FAIXA;
            V_VRALIQSIMPLESFAIXAAUX := V_VRALIQSIMPLES1FAIXA;
            V_VRPARCDEDUZIRFAIXAAUX := V_VRPARCDEDUZIR1FAIXA;
            V_VRALIQIMPOSTOFAIXAAUX := V_VRALIQIMPOSTO1FAIXA;
            V_VRPERRATISSFAIXAAUX := V_VRPERRATISS1FAIXA;
            V_VRALIQEFETIVAFAIXAAUX := V_VRALIQEFETIVA1FAIXA;
         ELSIF V_FAIXAENQUADRADA = 2 THEN
            V_CDFAIXAAUX := V_CD2FAIXA;
            V_VRALIQSIMPLESFAIXAAUX := V_VRALIQSIMPLES2FAIXA;
            V_VRPARCDEDUZIRFAIXAAUX := V_VRPARCDEDUZIR2FAIXA;
            V_VRALIQIMPOSTOFAIXAAUX := V_VRALIQIMPOSTO2FAIXA;
            V_VRPERRATISSFAIXAAUX := V_VRPERRATISS2FAIXA;
            V_VRALIQEFETIVAFAIXAAUX := V_VRALIQEFETIVA2FAIXA;
         ELSIF V_FAIXAENQUADRADA = 3 THEN
            V_CDFAIXAAUX := V_CD3FAIXA;
            V_VRALIQSIMPLESFAIXAAUX := V_VRALIQSIMPLES3FAIXA;
            V_VRPARCDEDUZIRFAIXAAUX := V_VRPARCDEDUZIR3FAIXA;
            V_VRALIQIMPOSTOFAIXAAUX := V_VRALIQIMPOSTO3FAIXA;
            V_VRPERRATISSFAIXAAUX := V_VRPERRATISS3FAIXA;
            V_VRALIQEFETIVAFAIXAAUX := V_VRALIQEFETIVA3FAIXA;
         ELSIF V_FAIXAENQUADRADA = 4 THEN
            V_CDFAIXAAUX := V_CD4FAIXA;
            V_VRALIQSIMPLESFAIXAAUX := V_VRALIQSIMPLES4FAIXA;
            V_VRPARCDEDUZIRFAIXAAUX := V_VRPARCDEDUZIR4FAIXA;
            V_VRALIQIMPOSTOFAIXAAUX := V_VRALIQIMPOSTO4FAIXA;
            V_VRPERRATISSFAIXAAUX := V_VRPERRATISS4FAIXA;
            V_VRALIQEFETIVAFAIXAAUX := V_VRALIQEFETIVA4FAIXA;
         ELSIF V_FAIXAENQUADRADA = 5 THEN
            V_CDFAIXAAUX := V_CD5FAIXA;
            V_VRALIQSIMPLESFAIXAAUX := V_VRALIQSIMPLES5FAIXA;
            V_VRPARCDEDUZIRFAIXAAUX := V_VRPARCDEDUZIR5FAIXA;
            V_VRALIQIMPOSTOFAIXAAUX := V_VRALIQIMPOSTO5FAIXA;
            V_VRPERRATISSFAIXAAUX := V_VRPERRATISS5FAIXA;
            V_VRALIQEFETIVAFAIXAAUX := V_VRALIQEFETIVA5FAIXA;
         ELSIF V_FAIXAENQUADRADA = 6 THEN
            V_CDFAIXAAUX := V_CD6FAIXA;
            V_VRALIQSIMPLESFAIXAAUX := V_VRALIQSIMPLES6FAIXA;
            V_VRPARCDEDUZIRFAIXAAUX := V_VRPARCDEDUZIR6FAIXA;
            V_VRALIQIMPOSTOFAIXAAUX := V_VRALIQIMPOSTO6FAIXA;
            V_VRPERRATISSFAIXAAUX := V_VRPERRATISS6FAIXA;
            V_VRALIQEFETIVAFAIXAAUX := V_VRALIQEFETIVA6FAIXA;
         END IF;

         --CALCULANDO O VALOR DA ALQUOTA DO IMPOSTO
         IF P_CDIMPOSTONFE IN ('00', '31') THEN
            IF V_RBA <= (P_VRSUBLIMITESN * 1.2) THEN
               IF V_RBA <= P_VRSUBLIMITESN THEN
                  IF (V_RBT12 >= 3600000) THEN
                     V_VRALIQIMPOSTO := V_VRALIQEFETIVA5FAIXA * (V_VRALIQIMPOSTO5FAIXA / 100);
                  ELSE
                     V_VRALIQIMPOSTO := V_VRALIQEFETIVAFAIXAAUX * (V_VRALIQIMPOSTOFAIXAAUX / 100);
                  END IF;
               ELSE
                  IF P_VRSUBLIMITESN = 3600000 THEN
                     V_VRALIQIMPOSTO := ((((3600000 * V_VRALIQSIMPLES5FAIXA / 100) - V_VRPARCDEDUZIR5FAIXA) / 3600000) * 100) * (V_VRALIQIMPOSTO5FAIXA / 100);
                  ELSIF P_VRSUBLIMITESN = 1800000 THEN
                     V_VRALIQIMPOSTO := ((((1800000 * V_VRALIQSIMPLES4FAIXA / 100) - V_VRPARCDEDUZIR4FAIXA) / 1800000) * 100) * (V_VRALIQIMPOSTO4FAIXA / 100);
                  END IF;
               END IF;
            END IF;
         ELSE
            V_VRALIQIMPOSTO := V_VRALIQEFETIVAFAIXAAUX * (V_VRALIQIMPOSTOFAIXAAUX / 100);
         END IF;

         IF P_CDIMPOSTONFE = 31 THEN
            --SEGUNDO A LEGISLAO A ALQUOTA DO ISS NO PODE ULTRAPASSAR OS 5%
            IF V_VRALIQIMPOSTO > 5 THEN
               V_VRALIQIMPOSTO := 5;
            END IF;
         ELSE
            --VERIFICANDO SE EXISTE A INCIDNCIA DO ISS NA OPERAO 
            BEGIN
               SELECT NVL(SUM(VRALIQSIMPLES),0)
                 INTO V_VRALIQIMPOSTOISS
                 FROM IMPSIMPLESFAIXA I, IMPOSTO II
                WHERE I.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
                  AND I.CDFAIXA = CASE WHEN V_FAIXAENQUADRADA IN (1, 2, 3, 4, 5) THEN V_CDFAIXAAUX WHEN V_FAIXAENQUADRADA IN (6) THEN V_CD5FAIXA END
                  AND I.CDIMPOSTO = II.CDIMPOSTO
                  AND II.CDIMPOSTONFE = 31;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_VRALIQIMPOSTOISS := 0;
            END;

            IF V_VRALIQIMPOSTOISS > 0 THEN
               --CALCULANDO A ALQUOTA DO ISS 
               IF V_FAIXAENQUADRADA IN (1, 2, 3, 4, 5) THEN
                  IF V_RBA <= (P_VRSUBLIMITESN * 1.2) THEN
                     V_VRALIQIMPOSTOISS := V_VRALIQEFETIVAFAIXAAUX * (V_VRALIQIMPOSTOISS / 100);
                     V_VRALIQIMPOSTOISSAUX := V_VRALIQIMPOSTOISS;
                  ELSE
                     V_VRALIQIMPOSTOISSAUX := V_VRALIQEFETIVAFAIXAAUX * (V_VRALIQIMPOSTOISS / 100);
                     V_VRALIQIMPOSTOISS := 0;
                  END IF;
               ELSIF V_FAIXAENQUADRADA IN (6) THEN
                  IF V_RBA <= (P_VRSUBLIMITESN * 1.2) THEN
                     IF V_RBA <= P_VRSUBLIMITESN THEN
                        IF (V_RBT12 >= 3600000) THEN
                           V_VRALIQIMPOSTOISS := V_VRALIQEFETIVA5FAIXA * (V_VRALIQIMPOSTOISS / 100);
                           V_VRALIQIMPOSTOISSAUX := V_VRALIQIMPOSTOISS;
                        ELSE
                           V_VRALIQIMPOSTOISS := V_VRALIQEFETIVAFAIXAAUX * (V_VRALIQIMPOSTOISS / 100);
                           V_VRALIQIMPOSTOISSAUX := V_VRALIQIMPOSTOISS;
                        END IF;
                     ELSE
                        IF P_VRSUBLIMITESN = 3600000 THEN
                           V_VRALIQIMPOSTOISS := ((((3600000 * V_VRALIQSIMPLES5FAIXA / 100) - V_VRPARCDEDUZIR5FAIXA) / 3600000) * 100) * (V_VRALIQIMPOSTOISS / 100);
                           V_VRALIQIMPOSTOISSAUX := V_VRALIQIMPOSTOISS;
                        ELSIF P_VRSUBLIMITESN = 1800000 THEN
                           V_VRALIQIMPOSTOISS := ((((1800000 * V_VRALIQSIMPLES5FAIXA / 100) - V_VRPARCDEDUZIR5FAIXA) / 1800000) * 100) * (V_VRALIQIMPOSTOISS / 100);
                           V_VRALIQIMPOSTOISSAUX := V_VRALIQIMPOSTOISS;
                        END IF;
                     END IF;
                  ELSE
                     V_VRALIQIMPOSTOISS := 0;
                     V_VRALIQIMPOSTOISSAUX := V_VRALIQIMPOSTOISS;
                  END IF;
               END IF;

               IF V_VRALIQIMPOSTOISS > 5 THEN
                  --BLOCO DESTINADO AO RATEIO DA PARTE QUE PASSA DOS 5% DA ALQUOTA DO ISS ENTRE OS OUTROS IMPOSTOS
                  IF V_FAIXAENQUADRADA IN (1, 2, 3, 4, 5) THEN
                     V_VRALIQIMPOSTO := V_VRALIQIMPOSTO + ((V_VRALIQIMPOSTOISS - 5) * (V_VRPERRATISSFAIXAAUX / 100));
                  ELSIF V_FAIXAENQUADRADA IN (6) THEN
                     V_VRALIQIMPOSTO := V_VRALIQIMPOSTO + ((V_VRALIQIMPOSTOISS - 5) * (V_VRPERRATISS5FAIXA / 100));
                  END IF;

                  V_VRALIQIMPOSTOISS := 5;
                  V_RATEIADIFISS := 'S';
               END IF;
            END IF;
         END IF;

         IF P_RATEIODIFERENCA = 'N' THEN
            --BLOCO DESTINADO AO CLCULO DA DIFERENA ENTRE A ALQUOTA EFETIVA ENCONTRADA E O SOMATRIO DAS ALQUOTAS ENCONTRADAS PARA CADA IMPOSTO QUE COMPOE A FAIXA DE FATURAMENTO
            --SEGUNDO A LEGISLAO, CASO EXISTA ESTA DIFERENA ESSE VALOR DEVER SER ACRESCIDO OU RETIRADO DO IMPOSTO DE MAIOR ALQUOTA NOMINAL
            BEGIN
               SELECT IM.CDIMPOSTONFE
                 INTO V_CDIMPOSTOMAXALIQUOTA
                 FROM IMPSIMPLESFAIXA I JOIN IMPOSTO IM ON (I.CDIMPOSTO = IM.CDIMPOSTO)
                WHERE I.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
                  AND I.CDFAIXA = V_CDFAIXAAUX
                  AND I.VRALIQSIMPLES = (SELECT MAX(I.VRALIQSIMPLES)
                                           FROM IMPSIMPLESFAIXA I JOIN IMPOSTO IM ON (I.CDIMPOSTO = IM.CDIMPOSTO)
                                          WHERE I.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
                                            AND I.CDFAIXA = V_CDFAIXAAUX
                                            AND IM.IDORGARRECAD = 'F'
                                            AND (((IM.CDIMPOSTONFE <> '31') AND (V_RATEIADIFISS = 'S')) OR (V_RATEIADIFISS = 'N')));
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_CDIMPOSTOMAXALIQUOTA := NULL;
            END;
            IF (V_CDIMPOSTOMAXALIQUOTA = P_CDIMPOSTONFE) THEN
               --A IDIA AQUI  CHAMAR ESSA MESMA PROCEDURE PARA CADA IMPOSTO QUE COMPE A FAIXA DE VALORES E A ALQUOTA ENCONTRADA PARA CADA UM SER ARREDONDADA
               --PARA DUAS CASAS DECIMAIS E ADICIONADO A UMA VARIVEL. DEPOIS O VALOR DESTA VARIVEL SER COMPARADA AO VALOR DA ALQUOTA EFETIVA ENCONTRADA
               --LEMBRANDO QUE O SOMATRIO DAS ALQUOTAS ENCONTRADAS PARA CADA IMPOSTO DEVER SER IDNTICO A ALQUOTA EFETIVA ENCONTRADA
               V_VRALIQSIMPLESRATEIO := 0;
               
               FOR C_RATEIO_IMPOSTO IN (SELECT IM.CDIMPOSTONFE, I.VRALIQSIMPLES, I.VRPERRATISS
                                          FROM IMPSIMPLESFAIXA I JOIN IMPOSTO IM ON (I.CDIMPOSTO = IM.CDIMPOSTO)
                                         WHERE I.CDSIMPLESNACIONAL = V_CDSIMPLESNACIONAL
                                           AND I.CDFAIXA = V_CDFAIXAAUX
                                           AND (((IM.CDIMPOSTONFE <> '31') AND (V_RATEIADIFISS = 'S')) OR (V_RATEIADIFISS = 'N'))
                                         ORDER BY IM.CDIMPOSTONFE)
               LOOP
                  CALCULA_ALIQUOTA_SN(P_CDEMPRESA, 
                                      P_CDTIPOOPER, 
                                      P_INIVALID, 
                                      P_CDFISOPEITEM, 
                                      C_RATEIO_IMPOSTO.CDIMPOSTONFE, 
                                      P_VRSUBLIMITESN,
                                      'S',
                                      V_RBT12VALIDA,
                                      V_RBAVALIDA,
                                      V_CDSIMPLESNACIONALVALIDA,
                                      V_CDFAIXAVALIDA,
                                      V_VRALIQUOTASNVALIDA);

                  V_VRALIQSIMPLESRATEIO := V_VRALIQSIMPLESRATEIO + ROUND(V_VRALIQUOTASNVALIDA, 2);
               END LOOP;

               IF V_RATEIADIFISS = 'S' THEN
                  IF V_FAIXAENQUADRADA = 6 THEN
                     IF (V_VRALIQEFETIVAFAIXAAUX + V_VRALIQIMPOSTOISSAUX) <> (V_VRALIQSIMPLESRATEIO + 5) THEN
                        V_VRALIQIMPOSTO := ROUND(V_VRALIQIMPOSTO, 2) + ((V_VRALIQEFETIVAFAIXAAUX + V_VRALIQIMPOSTOISSAUX) - (V_VRALIQSIMPLESRATEIO + 5));
                     END IF;
                  ELSE
                     IF (V_VRALIQEFETIVAFAIXAAUX - 5) <> V_VRALIQSIMPLESRATEIO THEN
                        V_VRALIQIMPOSTO := ROUND(V_VRALIQIMPOSTO, 2) + ((V_VRALIQEFETIVAFAIXAAUX - 5) - V_VRALIQSIMPLESRATEIO);
                     END IF;
                  END IF;
               ELSE
                  IF V_RBA <= (P_VRSUBLIMITESN * 1.2) THEN
                     IF V_VRALIQEFETIVAFAIXAAUX <> V_VRALIQSIMPLESRATEIO THEN
                        V_VRALIQIMPOSTO := ROUND(V_VRALIQIMPOSTO, 2) + (V_VRALIQEFETIVAFAIXAAUX - V_VRALIQSIMPLESRATEIO);
                     END IF;
                  ELSE
                     IF (V_VRALIQEFETIVAFAIXAAUX - V_VRALIQIMPOSTOISSAUX) <> V_VRALIQSIMPLESRATEIO THEN
                        V_VRALIQIMPOSTO := ROUND(V_VRALIQIMPOSTO, 2) + (V_VRALIQEFETIVAFAIXAAUX - V_VRALIQIMPOSTOISSAUX - V_VRALIQSIMPLESRATEIO);
                     END IF;
                  END IF;
               END IF;

            END IF;
         END IF;
         --PREENCHENDO OS PARMETROS DE SADA DA PROCEDURE
         P_RBT12 := V_RBT12;
         P_RBA := V_RBA;
         P_CDSIMPLESNACIONAL := V_CDSIMPLESNACIONAL;
         P_CDFAIXA := V_CDFAIXAAUX;
         P_VRALIQUOTASN := NVL(V_VRALIQIMPOSTO, 0);
      END IF;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CALCULA_ALIQUOTA_SN','6.05.xxx'); END;
/
