CREATE SEQUENCE USR_NRITEMSOLI START WITH 1;
/
CREATE TABLE USR_ITEMSOLI_SHA (
  NRITEMSOLI  NUMBER(10) DEFAULT USR_NRITEMSOLI.NEXTVAL    NOT NULL,
  CDFILISOLI  VARCHAR2(4)  NOT NULL,
  CDPRODUTO   VARCHAR2(20) NOT NULL,
  SGUNIDMED   VARCHAR2(10) NOT NULL,
  DTPROGENTR  DATE NOT NULL,
  QTSOLIPROD FLOAT NOT NULL,
  DTEXPORTADO DATE NULL,
  DTINCLUSAO DATE, 
  DTULTATU DATE, 
  NRORGINCLUSAO NUMBER, 
  CDOPERINCLUSAO VARCHAR2(12), 
  NRORGULTATU NUMBER, 
  NRORG NUMBER DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NOT NULL
  );
/
ALTER TABLE USR_ITEMSOLI_SHA ADD (
  CONSTRAINT USR_ITEMSOLI_SHA_PK PRIMARY KEY (NRITEMSOLI)
  );
/
ALTER TABLE ITEMSOLI ADD USR_SHA_INTEGRADO VARCHAR2(1) DEFAULT 'N'
   CONSTRAINT CKC_USR_SHA_INT_FORN CHECK ((USR_SHA_INTEGRADO IN ('S','N')) OR (USR_SHA_INTEGRADO IS NULL))
/
CREATE OR REPLACE TRIGGER USR_ATU_ITEMSOLI_SHA
BEFORE DELETE OR UPDATE OF IDSOLIAPROV
ON ITEMSOLI FOR EACH ROW
DECLARE
 PRAGMA AUTONOMOUS_TRANSACTION;
 V_NOME     VARCHAR2(1000);
 V_COMANDO  VARCHAR2(2000);
BEGIN
   V_NOME := 'EXPORTASOLCOMPRA';
   V_COMANDO := 'BEGIN USR_EXPORTA_SOLCOMP('||CHR(39)||V_NOME||CHR(39)||'); END;';
   IF UPDATING THEN
      IF (NVL(:NEW.IDSOLIAPROV, 'N') <> NVL(:OLD.IDSOLIAPROV, 'N')) THEN
         :NEW.USR_SHA_INTEGRADO   := 'N';
      END IF;
   END IF;
   IF DELETING THEN
      IF ((NVL(:OLD.IDSOLIAPROV, 'N') = 'N') AND (NVL(:OLD.USR_SHA_INTEGRADO, 'N') = 'N')) THEN
         RAISE_APPLICATION_ERROR(-20501, 'Operação Bloqueada: Item não pode ser excluído antes de ser integrado.');
      END IF;
   END IF;
   
   BEGIN
      DBMS_SCHEDULER.DROP_JOB(V_NOME);
    EXCEPTION
      WHEN OTHERS THEN
            NULL;
   END;
   BEGIN
      DBMS_SCHEDULER.CREATE_JOB (
          JOB_NAME => V_NOME,
          JOB_TYPE => 'PLSQL_BLOCK',
          JOB_ACTION => V_COMANDO,
          START_DATE => SYSDATE + INTERVAL '60' SECOND,
          ENABLED => TRUE    
      );
   END;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('USR_ATU_ITEMSOLI_NGD','6.10.xxx'); END;
/
/*
-- inserts para teste
Insert into USR_ITEMSOLI_SHA (CDFILISOLI,CDPRODUTO, SGUNIDMED,DTPROGENTR,QTSOLIPROD) values ('0003','1160101900','UN', to_date('01/02/28','DD/MM/RR'),'7');
Insert into USR_ITEMSOLI_SHA (CDFILISOLI,CDPRODUTO, SGUNIDMED,DTPROGENTR,QTSOLIPROD) values ('0003','1161600704','UN', to_date('01/02/28','DD/MM/RR'),'1');
Insert into USR_ITEMSOLI_SHA (CDFILISOLI,CDPRODUTO, SGUNIDMED,DTPROGENTR,QTSOLIPROD) values ('0005','1261600102','UN', to_date('28/07/48','DD/MM/RR'),'20');
Insert into USR_ITEMSOLI_SHA (CDFILISOLI,CDPRODUTO, SGUNIDMED,DTPROGENTR,QTSOLIPROD) values ('0005','1261600702','UN', to_date('28/07/48','DD/MM/RR'),'1');
Insert into USR_ITEMSOLI_SHA (CDFILISOLI,CDPRODUTO, SGUNIDMED,DTPROGENTR,QTSOLIPROD) values ('0003','1160103401','UN', to_date('01/02/28','DD/MM/RR'),'6,026');
*/
/
